/*syshead.h*/
#ifndef __SYS_HEAD_H__
#define __SYS_HEAD_H__

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>
#include <time.h>

typedef  unsigned int   F_U32;
typedef  signed int     F_S32;
typedef  signed short	F_S16;
typedef  unsigned short F_U16;
typedef  unsigned char	F_U8;
typedef  signed char	F_S8;


#define	O_RDONLY		0		/* +1 == FREAD	*/
#define	O_WRONLY		1		/* +1 == FWRITE	*/
#define	O_RDWR			2		/* +1 == FREAD|FWRITE */
#define	O_CREATE		3

#define FILE_SYSTEM_SIZE    (16 * 1024 * 1024)     					/* 16M */
#define FILE_DIRECT_SIZE    (512 * 1024)							/* 512K */
#define EACH_DIRECT_SIZE     sizeof(struct F_Dir)					/*  the direct struct  size */
#define FILE_DIRECT_NUM     ((int)(FILE_DIRECT_SIZE / EACH_DIRECT_SIZE))	/*  */
#define BLOCK_SIZE 			(2 * 1024)								/* 2K */
#define BLOCK_NUM			((int)(FILE_SYSTEM_SIZE / BLOCK_SIZE))         /* 8K */   
#define FILE_BITMAP_SIZE    (2 * BLOCK_NUM)			    /* 16K-> 16 bit express 1 block */
#define FILE_DATA_START		(FILE_DIRECT_SIZE + FILE_BITMAP_SIZE)
#define FS_PATH_LENTH		67
#define FS_FNAME_LENTH 		12
#define F_SIZE_MAX   		(1024 * 4)// (FILE_SYSTEM_SIZE - FILE_DIRECT_SIZE - FILE_BITMAP_SIZE) /*the max file size*/

#define BITMAP_UNUSED 		0x0
#define BITMAP_LAST_BLOCK 	0xFFFF

/* ļĿ¼ݽṹ */
struct F_Dir
{
	F_U8		file_name[FS_FNAME_LENTH];			/*File's Name*/
	F_U32		file_startbl;						/*Directory's start logic block*/
	F_U32		file_size;							/*File's size in directory*/
	char createTime[50]; 								/*store the current time*/
	char writeTime[50];
};


/* ڵݽṹ */
struct F_Node 
{
	F_S32 	f_flags;						/* File's open mode*/
	F_S32 	f_nodeid;					/* File Node ID*/
	struct 	F_Dir	 f_dir;				/* File's directory info*/
	F_U32   i_blocks;					/* ļĿ */
	
};

#define TRUE 	1
#define FALSE	0

#define OP_SUCCESS	1
#define OP_FAILED	-1

#define INVALID_FID	-1
#define SEEK_SUCCESS	1
#define SEEK_FAILED	-1

static inline F_U32 get_block_num(F_U32 file_size)
{
	if ((file_size % BLOCK_SIZE) == 0)
	{
		return (file_size/BLOCK_SIZE);
	}
	else
	{
		return ((F_U32)(file_size/BLOCK_SIZE) + 1);
	}
}
#endif /*__SYS_HEAD_H__*/

/* globalvar.h*/
#ifndef __GLOBALVAR_H__
#define __GLOBALVAR_H__

extern char bit_map_buf[];
extern  const char file_system_name[];

/*  fp points the file_system */
extern  FILE *fp; 
/* openfileno is the index of fp_array,the opend file */
extern F_U32 openfileno ;
/* debugresult is used to debug the mid_result */
extern F_S32 debugresult;
extern  F_U32 filenums ;
extern  struct F_Node mid_node;
extern  struct F_Node *f_node_ptr ;
extern  struct F_Node * fnode_table[];
extern  int free_block_num;
#endif


/* globalvar.c*/
#include "syshead.h"

/* the datas of bit_map in the file system is stored with bit_map_buf */
char bit_map_buf[FILE_BITMAP_SIZE];
const char file_system_name[] = "filesys.dat";

/*  fp points the file_system */
FILE *fp = NULL; 

/* openfileno is the index of fp_array,the opend file */
F_U32 openfileno = 0;

/* debugresult is used to debug the mid_result */
F_S32 debugresult;
F_U32 filenums = 0;

struct F_Node * fnode_table[FILE_DIRECT_NUM] = {NULL,};

struct F_Node *f_node_ptr = NULL;

int free_block_num;

/* admin.h*/
#ifndef __ADMIN_H__
#define __ADMIN_H__

#include "syshead.h"

F_S32 write_back_dir(int i);
F_S32 write_back_bitmap();
F_S32 get_free_block_num();
F_S32 get_next_free_block(int curr_block_num);
F_S8 check_file_mode(char *mode);
#endif

/* admin.c*/
#include "Syshead.h"
#include "globalvar.h"
/* init the bitmap */
extern struct F_Super_Block *f_Super_Block ;


F_S32 write_back_dir(int i);
F_S32 write_back_bitmap();
F_S32 get_free_block_num();
F_S32 get_next_free_block(int curr_block_num);

/*write back dir to the file system*/
F_S32 write_back_dir(int i)
{
	int writed;
	debugresult = fseek(fp, FILE_BITMAP_SIZE + i * sizeof (struct F_Dir), SEEK_SET);
	writed = fwrite(&(fnode_table[i]->f_dir), 1, sizeof (struct F_Dir), fp);
	if (writed  != sizeof (struct F_Dir))
	{
		printf("\nwrite back directory error!\n");
		return OP_FAILED;
	}

	fflush(fp);
	return OP_SUCCESS;
}

F_S32 write_back_bitmap()
{
	int writed;
	debugresult = fseek(fp, 0, SEEK_SET);
	writed = fwrite(bit_map_buf, 1, FILE_BITMAP_SIZE, fp);
	if (writed  != FILE_BITMAP_SIZE)
	{
		printf("\nwrite back bitmap error!\n");
		return OP_FAILED;
	}
	return writed;
	fflush(fp);
}
F_S32 get_free_block_num()
{
	unsigned short * p_bit_map = (unsigned short * )bit_map_buf;
	int i, free_num = 0;

	for (i = 0 ; i < BLOCK_NUM; i++)
	{
		if (p_bit_map[i] == BITMAP_UNUSED)
		{
			free_num++;
		}
	}

	return free_num;
}

F_S32 get_next_free_block(int curr_block_num)
{
	unsigned short * p_bit_map = (unsigned short * )bit_map_buf;
	unsigned short * start;
	int total_remainder;
	int i;

	if (curr_block_num > BLOCK_NUM)
	{
		return -1;
	}

	/* start from the current block bitmap */	
	for (i = curr_block_num; i < BLOCK_NUM ; i ++)
	{
		if (p_bit_map[i] == BITMAP_UNUSED)
		{
			curr_block_num = i;
			return curr_block_num;
		}
	}
	if (i > BLOCK_NUM)
	{
		return -1;
	}
	
}

/* get the current system time and date function */
int getCurrentTime(char *currtime)
{
	time_t now;
	time(&now);
	printf("The time is :");
	//puts(asctime(localtime(&now)));
	strcpy(currtime,asctime(localtime(&now)));
	printf("currtime is %s:\n",currtime);
	return OP_SUCCESS;
}


/* filesys.c*/
#include "syshead.h"
#include "globalvar.h"
#include "admin.h"

static void create_file_system();
F_S8 open_file_system();
/*struct F_Node *f_open(F_U8 *file_name, F_S32 flags,...);*/
F_S32 f_open(F_U8 *file_name, F_S32 flags);
/*F_S32 create_a_file(F_U8 *filename, F_U32 mode); */
F_S32 f_create(F_U8 *file_name, F_U32 mode);
F_S32 f_write(F_U8 *buf, F_U32 nbyte, F_S32 f_id);
void print_help();
F_S32 f_read(F_U8 *buf, F_U32 nbyte, F_S32 f_id);

struct F_Node *p_F_Node = NULL;       /* ָڵָ */
struct F_Super_Block *f_Super_Block = NULL; /* ָ򳬼ڵָ */

/* create a file system to emulate a disk file system */
void create_file_system()
{
	/* len ,the recycle times ,for init the new file system with 0 */
	int len;
	
	/* Open a filesystem for writing, if filesystem existed, we will truncate it into zero */
	fp = fopen(file_system_name, "r+");
	if (fp == NULL)
	{
		printf("Create file system error!  \n");
		exit(1);
	}

	/* init file system , bitmap[i]=0x00(unused), bitmap[i]=0xffff(last block)*/
	for (len = 0 ; len < FILE_SYSTEM_SIZE; len ++)	
	{
		fputc(0, fp);
	}

	fflush(fp);

	printf("Create file system success!  \n");
}

/* open a file system */
F_S8 open_file_system()
{
	/* i, the recycle times ,for read the each dir to the dir_array*/
	int i ;
	char idx;
	F_U8 fnode_buf[sizeof(struct F_Dir)];
	struct F_Node *p_node;


	if(fp == NULL)
	{
		fp = fopen(file_system_name, "r+");
		if (fp == NULL)
		{
			create_file_system();
		}
	}
	/* read the bit_map datas to the bit_map_buf */
	debugresult = fseek(fp, 0, SEEK_SET);
	if (debugresult == -1)
	{
		printf("\n fseek the head of the file system fail! \n");
		return OP_FAILED;
	}

	debugresult = fread(bit_map_buf, 1, FILE_BITMAP_SIZE, fp);
	if (debugresult <  FILE_BITMAP_SIZE)
	{
		printf("\n fread bit map datas fail! \n");
		return OP_FAILED;
	}

	/* read all dirs from the file system to the memery struct F_node list */
	for (i = 0 ; i < FILE_DIRECT_NUM - 1; i++)
	{
		/* fp now point the start of file directory */
		debugresult = fseek(fp, 0, SEEK_CUR);
		if (debugresult == -1)
		{
			printf("\n fseek fail! \n");
			return SEEK_FAILED;
		}
		
		debugresult = fread(fnode_buf, 1, sizeof(struct F_Dir), fp);
		if (debugresult < sizeof(struct F_Dir))
		{
			printf("\n fread fail! \n");
			return OP_FAILED;
		}
			
		//if (strcmp(next->f_dir.file_name,""))
		if (strcmp(((struct F_Dir *)(fnode_buf))->file_name,"") == 0)
		{
			continue;
		}

		p_node = (struct F_Node *)malloc(sizeof(struct F_Node));
		if (p_node == NULL)
		{
			 printf("\n malloc fail! \n");
			return OP_FAILED;
		}

		memset(p_node, 0x0, sizeof(struct F_Node));
		/* count the dir numbers with filenums */
		filenums ++;
		/* init each F_Node information */
		memcpy(&(p_node->f_dir), fnode_buf, sizeof(struct F_Dir));
		p_node->f_nodeid = i;
		p_node->i_blocks = get_block_num(p_node->f_dir.file_size);
		p_node->f_flags = O_RDWR;
		fnode_table[i] = p_node;
	}

	/* now fp points the end of dir section */
	printf("i = %d\n",i);
	printf("filenums = %d\n",filenums);
	printf("open file system success\n",i);
	return OP_SUCCESS;
}

/* create a file */
F_S32 f_create(F_U8 *file_name, F_U32 flags)
{
	int i, ret;
	struct F_Node * ptr;
	
	if (flags != O_CREATE)
	{
		printf("can't create file in such flag\n");
		return INVALID_FID;
	}

	/* search fnode table for a valid position */
	for (i = 0 ; i < FILE_DIRECT_NUM; i++)
	{
		if (fnode_table[i] == NULL)
		{
			/* get the position */
			ptr = (struct F_Node  *) malloc(sizeof (struct F_Node));
			if (ptr == NULL)
			{
				printf("can't malloc space\n");
				return INVALID_FID;
			}
			memset(ptr, 0x0, sizeof(struct F_Node));
			
			fnode_table[i] = ptr;

			/* fill ptr */
			strcpy(ptr->f_dir.file_name, file_name);
			ptr->f_dir.file_size = 0;
			ptr->f_dir.file_startbl = 0;
			getCurrentTime(ptr->f_dir.createTime);
			getCurrentTime(ptr->f_dir.writeTime);
			//printf("the created  file's current time is : %s",ptr->f_dir.createTime);
			ptr->f_flags = flags;
			ptr->f_nodeid = i;
			ptr->i_blocks = 0;

			if ((ret = write_back_dir(i)) != OP_SUCCESS)
			{
				printf("can't write back dir\n");
				return INVALID_FID;		
			}

			printf("create file success!\n");
			return i;
		}
	}

	return INVALID_FID;
}


/* open a file :F_U32 mode */
F_S32 f_open(F_U8 *file_name, F_S32 flags)
{

	struct F_Node * nextNode;
	int i = 0; /* the location of each fp_array */
	
	if (file_name == NULL)
	{
		printf("\n please input the file name! \n");
		return OP_FAILED;
	}

	if ((flags != O_RDWR) && (flags != O_CREATE))
	{
		printf("invalid flag!\n");
	}


	for (i = 0; i < FILE_DIRECT_NUM; i++)
	{
		if (fnode_table[i] != NULL)
		{
			/* look up whether file is existed */
			if (strcmp(file_name, fnode_table[i]->f_dir.file_name) == 0)
			{
				printf("open file success!\n");
				return i;
			}
		}
	}


	if (flags == O_CREATE)
	{
		/* call file create routine*/
		return f_create(file_name, flags);
	}
	else
	{
		printf("open non-existed file!\n");
		return -1;
	}

}

/* buf is used to store the file datas,
 * nbyte is the number bytes of reading, 
 * f_id is the file index in the fnode_table.
 */
F_S32 f_read(F_U8 *buf, F_U32 nbyte, F_S32 f_id)
{
	struct F_Node * p_node;
	int block_num, i, remainer, actual_block, need_proc_last_block;
	int start_block, next_block;
	unsigned short * p_bit_map = (unsigned short *)bit_map_buf;
	F_U8 *buf_ptr = buf;
	int byte_readed, total_byte_read = 0;

	
	if ((nbyte < 0) || (buf == NULL) )
	{
		printf("param error in reading !\n");
		return 0;
	}

	if ((f_id < 0) || (f_id >= FILE_DIRECT_NUM))
	{
		printf("param over range !\n");
		return 0;
	}

	p_node = fnode_table[f_id];
	if (p_node == NULL)
	{
		printf("file not existed !\n");
		return 0;
	}
	start_block = p_node->f_dir.file_startbl;
	
	if (nbyte <= p_node->f_dir.file_size)
	{
		/* read nbyte */
		block_num = get_block_num(nbyte);
		remainer = nbyte % BLOCK_SIZE;
	}
	else
	{
		/* read file param */
		block_num = p_node->i_blocks;
		remainer = p_node->f_dir.file_size % BLOCK_SIZE;
	}

	if (remainer == 0)
	{
		actual_block = block_num;
		need_proc_last_block = FALSE;
	}
	else
	{
		actual_block = block_num - 1;
		need_proc_last_block = TRUE;
	}

	
	for (i = 0; i < actual_block ; i++)
	{
		next_block = p_bit_map[start_block];

		/* block index begin from 1, directory begin from 0 */
		debugresult = fseek(fp, FILE_DATA_START + (start_block - 1) * BLOCK_SIZE, SEEK_SET);
		if (debugresult != 0)
		{
			printf("can't seek file pointer !\n");
			return 0;
		}

		/* read file */
		byte_readed = fread(buf_ptr, 1, BLOCK_SIZE, fp);
		if (byte_readed < BLOCK_SIZE)
		{
			printf("read file error !\n");
			return 0;
		}

		total_byte_read += byte_readed;

		buf_ptr += BLOCK_SIZE;
		
		if (p_bit_map[start_block] == BITMAP_LAST_BLOCK)
		{
			/* break */
			break;
		}
		else
		{
			start_block = next_block;
		}
		
	}


	if (need_proc_last_block)
	{
		/* get last block num */
	//	next_block = p_bit_map[start_block];

		debugresult = fseek(fp, FILE_DATA_START + (start_block - 1) * BLOCK_SIZE, SEEK_SET);
		if (debugresult != 0)
		{
			printf("can't seek file pointer !\n");
			return 0;
		}

		/* read file */
		byte_readed = fread(buf_ptr, 1, remainer, fp);
		if (byte_readed < remainer)
		{
			printf("read file error !\n");
			return 0;
		}

		total_byte_read += byte_readed;
	}
	else
	{
		/* don't process*/
	}

	printf("read file success !\n");
	return total_byte_read;
}

/* write a file ,and store the file data to the file system*/
/* buf  stores the file datas to write to the file system,
 * nbyte is the number bytes of writing, 
 * f_id is the file index in the fnode_table.
 */
F_S32 f_write(F_U8 *buf, F_U32 nbyte, F_S32 f_id)
{
	struct F_Node * p_node;
	int block_num, i, remainer, actual_block, need_proc_last_block;
	int prev_block, next_block, free_block;
	unsigned short * p_bit_map = (unsigned short *)bit_map_buf;
	F_U8 *buf_ptr = buf;
	int byte_written, total_byte_written = 0;
	

	/* curren block num index is beginning from 0 */
	int curr_block_num = 0;

	
	if ((nbyte < 0) || (buf == NULL) )
	{
		printf("param error in writing !\n");
		return 0;
	}

	if ((f_id < 0) || (f_id >= FILE_DIRECT_NUM))
	{
		printf("param over range !\n");
		return 0;
	}

	p_node = fnode_table[f_id];
	if (p_node == NULL)
	{
		printf("file not existed !\n");
		return 0;
	}

	/***************************************/
	getCurrentTime(p_node->f_dir.writeTime);
	block_num = get_block_num(nbyte);
	remainer = nbyte % BLOCK_SIZE;
	
	free_block = get_free_block_num();
	if (free_block < block_num)
	{
		actual_block = free_block;
		need_proc_last_block = FALSE;
	}
	else
	{
		if (remainer == 0)
		{
			actual_block = block_num;
			need_proc_last_block = FALSE;
		}
		else
		{
			actual_block = block_num - 1;
			need_proc_last_block = TRUE;
		}
	}
	
#if 1
	
	/* find the first free block index */
	curr_block_num = get_next_free_block(curr_block_num);
	if (curr_block_num == -1)
	{
		printf("\nno free block remain\n");
		return 0;
	}
	
	p_node->f_dir.file_startbl = curr_block_num + 1;
		
	if (actual_block != 0)
	{
		for (i = 0; i < actual_block; i++)
		{
			int next_i;
			prev_block = curr_block_num;
			
			/* block index begin from 1, directory begin from 0 */
			debugresult = fseek(fp, FILE_DATA_START + (curr_block_num) * BLOCK_SIZE, SEEK_SET);
			if (debugresult != 0)
			{
				printf("can't seek file pointer !\n");
				return 0;
			}
			/* write file */
			byte_written = fwrite(buf_ptr, 1, BLOCK_SIZE, fp);
			if (byte_written < BLOCK_SIZE)
			{
				printf("write file error !\n");
				return 0;
			}
			p_bit_map[prev_block] = BITMAP_LAST_BLOCK;
			buf_ptr += BLOCK_SIZE;
			total_byte_written += byte_written;

			next_i = i+1;
			if (next_i < actual_block)
			{
				curr_block_num = get_next_free_block(curr_block_num + 1);
				if (curr_block_num == -1)
				{
					printf("\nno free block remain\n");
					return 0;
				}

				p_bit_map[prev_block] = curr_block_num + 1;
			}
		}

		/* need to deal the last nbyte which doesn't fill fully in the last block */
		if (need_proc_last_block)
		{
			/* get last block num */
			
	 		curr_block_num = get_next_free_block(curr_block_num + 1);
			if (curr_block_num == -1)
			{
				printf("\nno free block remain\n");
				return 0;
			}
			p_bit_map[prev_block] = curr_block_num + 1;
			p_bit_map[curr_block_num] = BITMAP_LAST_BLOCK;

			debugresult = fseek(fp, FILE_DATA_START + (curr_block_num) * BLOCK_SIZE, SEEK_SET);
			if (debugresult != 0)
			{
				printf("can't seek file pointer !\n");
				return 0;
			}

			/* write file */
			byte_written = fwrite(buf_ptr, 1, remainer, fp);
			if (byte_written < remainer)
			{
				printf("write file error !\n");
				return 0;
			}

			total_byte_written += byte_written;
		}
		else
		{
			/* don't process*/
		}
	}
	else
	{
		/* only one block will write , and the actual byte is less than the block size */
		/* block index begin from 1, directory begin from 0 */
		debugresult = fseek(fp, FILE_DATA_START + (curr_block_num) * BLOCK_SIZE, SEEK_SET);
		if (debugresult != 0)
		{
			printf("can't seek file pointer !\n");
			return 0;
		}
		/* write file */
		byte_written = fwrite(buf_ptr, 1, remainer, fp);
		if (byte_written < remainer)
		{
			printf("write file error !\n");
			return 0;
		}
		p_bit_map[curr_block_num] = BITMAP_LAST_BLOCK;
		total_byte_written = remainer;
	}

	p_node->f_dir.file_size = total_byte_written;
	p_node->i_blocks = block_num;
	/* write back the file direct to the file system */
	/** f_id is the index of the fnode_table */
	debugresult = write_back_dir(f_id);
	if (debugresult == OP_FAILED)
	{
		printf("write file dir failed !\n");
		return 0;
	}

	debugresult = write_back_bitmap();
	if (debugresult == OP_FAILED)
	{
		printf("write file bitmap failed !\n");
		return 0;
	}
	
	fflush(fp);
	printf("write file success !\n");
	return total_byte_written;
	
	/********************************************************/
#endif

}

/* delete a file from the file system */
F_S32 f_del(char *file_name)
{
	F_S32 f_id;
	int i,j;
	F_S32 currblk_num_index,nextblk_num;
	F_U32 totalblk_num;
	unsigned short * p_bitmap_buf = (unsigned short *)bit_map_buf;
	
	for (i = 0; i < FILE_DIRECT_NUM ; i++)
	{
		if (fnode_table[i] != NULL)
		{
			/* the file is existing */
			if (strcmp(fnode_table[i]->f_dir.file_name, file_name) == 0)
			{
				f_id = i;
				/* currblk_num is the current file block number */
				currblk_num_index = fnode_table[i]->f_dir.file_startbl - 1;
				totalblk_num = fnode_table[i]->i_blocks;
				
				/* delete the block numbers in the bit_map_buf */
				for (j = 0 ; j < totalblk_num ;  j ++)
				{
					/* nextblk_num_index is the next file block number */
					nextblk_num = p_bitmap_buf[currblk_num_index];
					p_bitmap_buf[currblk_num_index] = BITMAP_UNUSED;
					currblk_num_index = nextblk_num -1;
				}

				/* write back the bit_map_buf to the file system */
				debugresult = write_back_bitmap();
				if (debugresult == OP_FAILED)
				{
					printf("write file bitmap failed !\n");
					return 0;
				}
			
			#if 1
				/*clear out the deleting file dir in the direct location */
				memset(&(fnode_table[f_id]->f_dir), 0x00, sizeof(struct F_Dir));
				debugresult = write_back_dir(f_id);
				if (debugresult == OP_FAILED)
				{
					printf("can't write back directory\n");
					return debugresult;
				}
				/* delete the file direct in the fnode_table */
				free(fnode_table[i]);
				fnode_table[i] = NULL;
			#endif
			
				printf("\n delete the file %s success! \n", file_name);
				return OP_SUCCESS;
			}
		}
	}
	/* the file is not existing */
	if (i >= FILE_DIRECT_NUM)
	{
		printf("\n the file %s is not existing! \n", file_name );
		printf("\n delete the file %s failed! \n", file_name );
		return OP_FAILED;
	}
}

/* list all of the files in the file system !*/
int f_list()
{
	int i;
	printf("\n list all of the files in the file system ! \n");
	for (i = 0; i < FILE_DIRECT_NUM; i++)
	{
		//if (strcmp(fnode_table[i]->f_dir.file_name,"") == 0)
		if(fnode_table[i] == NULL)
		{
			continue;
		}
		printf("###########################\n");
		printf("File Name:%s\n", fnode_table[i]->f_dir.file_name);
		printf("File size:%d\n", fnode_table[i]->f_dir.file_size);
		printf("Create time:%s",fnode_table[i]->f_dir.createTime);
		printf("Write time:%s",fnode_table[i]->f_dir.writeTime);
	}

	printf(" list all files end! \n");
	return OP_SUCCESS;
}

void print_help()
{
	printf("Please select operations: \n");
	printf("		1. Create a new file system \n");
	printf("		2. Make a new file \n");
	printf("		3. Del a file \n");
	printf("		4. List files \n");
	printf("		5. Write datas to a file \n");
	printf("		6. Read datas from a file \n");
	printf("		7. Exit \n");
	printf("	    		\n");
	printf("		h. for help \n");
}

int main()
{
	/* fname_buf stores the file name */
	char fname_buf[FS_FNAME_LENTH];
	/* the operation choice key */
	char key;
	/* the file mode */
	F_U32 mode;
	/*store the result of the operation successful or failed in the ret*/
	int ret;
	/* the stored datas in the fdata_buf*/
	char fdata_buf[F_SIZE_MAX];	
	/* the numbers of the stored bytes of the files*/
	F_U32 nbyte;
	/* f_id is the file index in the fnode_table of the file name */
	F_S32 f_id;
	/* byte_op is the number of bytes of the reading/writing */
	int byte_op;
	FILE * debugf;
	print_help();

	key = '0';
	/*if the file system is existing, open file system firstly */
	open_file_system();	

	while (key != '7')
	{
		key = getchar();
		putchar(key);
		/* clean the fname_buf and fdata_buf with 0 */
		memset(fname_buf, 0x00, FS_FNAME_LENTH);
		memset(fdata_buf, 0x00, F_SIZE_MAX);
		
		switch (key)
		{
			case '1':
				/* create a new file system !*/
				create_file_system();
				/* open the file system */
				open_file_system();	
				break;
			case '2':
				puts("\n Please input a new file name:");
				scanf("%s", fname_buf);
				scanf("%d", &mode);
				/*create a new empty file */
				f_open(fname_buf, mode);
				break;
			case '3':
				puts("\n Please input a deleting file name:");
				scanf("%s", fname_buf);
				/* delete the file with the name in fname_buf */
				ret = f_del(fname_buf);
				break;
			case '4':
				/* list all of the files in the file system */
				ret = f_list();
				break;
			case '5':
				puts("\n Please input a writing file name:\n");
				scanf("%s", fname_buf);
				puts("Please input datas to stored:\n");
				scanf("%s", fdata_buf);
				f_id = f_open(fname_buf, O_CREATE);
				byte_op = f_write(fdata_buf, strlen(fdata_buf) + 1, f_id);
				ret = f_list();
				break;
			case '6':
				puts("Please input a reading file name:\n");
				scanf("%s", fname_buf);
				/* find the index in fnode_table of the reading file with name in fname_buf*/
				f_id = f_open(fname_buf, O_CREATE);
				/*clean the fdata buf to 0 */
				memset(fdata_buf, 0x00, F_SIZE_MAX);
				/* reading datas to the fdata_buf */
				byte_op = f_read(fdata_buf, F_SIZE_MAX, f_id);		
				printf(" the datas of the file %s:\n",fname_buf);
				puts(fdata_buf);
				break;
			case 'h':
				printf("\n");
				print_help();
				break;
			case '7':
				/* close the open file system which is stored in the fp */
				if (fp != NULL)
				{
					fclose(fp);
				}
				break;
			default:
				printf("\n");
		}
	}
	return 0;
}
